@@ -23,7 +23,7 @@
 #include "go-data-impl.h"
 #include <goffice/utils/go-format.h>
 #include <goffice/utils/go-locale.h>
-#include <goffice/utils/go-math.h>
+#include <goffice/math/go-math.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
