@@ -31,6 +31,13 @@
 #include <string.h>
 #include <stdlib.h>
 
+/* GogGrid object was used with "Grid" role name by GogChart. We've changed the role
+ * name to "Backplane", since that's what the user see in the UI, and that's what GogGrid
+ * really is. We do the substitution on save/load for backward compatibility. */
+
+#define GOG_BACKPLANE_OLD_ROLE_NAME	"Grid"
+#define GOG_BACKPLANE_NEW_ROLE_NAME	"Backplane"
+
 GType
 gog_persist_get_type (void)
 {
@@ -272,7 +279,7 @@ gog_dataset_sax_save (GogDataset const *set, GsfXMLOut *output)
 void
 gog_object_write_xml_sax (GogObject const *obj, GsfXMLOut *output)
 {
-	gint	     n;
+	guint	     n;
 	GParamSpec **props;
 	GSList	    *ptr;
 
@@ -281,8 +288,12 @@ gog_object_write_xml_sax (GogObject const *obj, GsfXMLOut *output)
 	gsf_xml_out_start_element (output, "GogObject");
 
 	/* Primary details */
-	if (obj->role != NULL)
-		gsf_xml_out_add_cstr (output, "role", obj->role->id);
+	if (obj->role != NULL) {
+		if (strcmp (obj->role->id, GOG_BACKPLANE_NEW_ROLE_NAME) == 0)
+			gsf_xml_out_add_cstr (output, "role", GOG_BACKPLANE_OLD_ROLE_NAME);
+		else
+			gsf_xml_out_add_cstr (output, "role", obj->role->id);
+	}
 	if (obj->explicitly_typed_role || obj->role == NULL)
 		gsf_xml_out_add_cstr (output, "type", G_OBJECT_TYPE_NAME (obj));
 
@@ -333,7 +344,11 @@ gog_object_new_from_xml (GogObject *parent, xmlNode *node)
 	if (role == NULL) {
 		g_return_val_if_fail (parent == NULL, NULL);
 	} else {
-		res = gog_object_add_by_name (parent, role, res);
+		/* FIXME */
+		if (strcmp (role, GOG_BACKPLANE_OLD_ROLE_NAME) == 0)
+			res = gog_object_add_by_name (parent, GOG_BACKPLANE_NEW_ROLE_NAME, res);
+		else
+			res = gog_object_add_by_name (parent, role, res);
 		xmlFree (role);
 	}
 
@@ -390,6 +405,9 @@ gogo_dim_start (GsfXMLIn *xin, xmlChar const **attrs)
 	xmlChar const *dim_str = NULL, *type_str = NULL;
 	GType type;
 
+	if (NULL == state->obj)
+		return;
+
 	g_return_if_fail (IS_GOG_DATASET (state->obj));
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
@@ -426,6 +444,10 @@ static void
 gogo_dim_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *unknown)
 {
 	GogXMLReadState *state = (GogXMLReadState *)xin->user_state;
+
+	if (NULL == state->obj)
+		return;
+
 	g_return_if_fail (IS_GOG_DATASET (state->obj));
 
 	if (NULL != state->dimension) {
@@ -446,6 +468,11 @@ gogo_prop_start (GsfXMLIn *xin, xmlChar const **attrs)
 	GType prop_type;
 	int i;
 
+	if (NULL == state->obj) {
+		state->prop_spec = NULL;
+		return;
+	}
+
 	for (i = 0; attrs != NULL && attrs[i] && attrs[i+1] ; i += 2)
 		if (0 == strcmp (attrs[i], "name"))
 			prop_str = attrs[i+1];
@@ -502,7 +529,7 @@ gogo_prop_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *unknown)
 	GType prop_type, prop_ftype;
 	GValue val = { 0 };
 
-	if (state->prop_spec == NULL)
+	if (state->obj == NULL || state->prop_spec == NULL)
 		return;
 
 	prop_type = G_PARAM_SPEC_VALUE_TYPE (state->prop_spec);
@@ -527,7 +554,7 @@ gogo_prop_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *unknown)
 			return;
 		}
 
-		if (!gsf_xml_gvalue_from_str (&val, prop_ftype, content)) {
+		if (!gsf_xml_gvalue_from_str (&val, prop_type, content)) {
 			g_warning ("could not convert string to type `%s' for property `%s'",
 				   g_type_name (prop_type), state->prop_spec->name);
 			return;
@@ -538,6 +565,7 @@ gogo_prop_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *unknown)
 	g_value_unset (&val);
 }
 
+/* NOTE : every path through this must push something onto obj_stack. */
 static void
 gogo_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
@@ -561,21 +589,23 @@ gogo_start (GsfXMLIn *xin, xmlChar const **attrs)
 		} else
 			res = g_object_new (t, NULL);
 
-		g_return_if_fail (res != NULL);
+		if (res == NULL) {
+			g_warning ("unknown type '%s'", type);
+		}
 	} else
 		res = NULL;
 
 	if (role != NULL) {
-		res = gog_object_add_by_name (state->obj, role, res);
-	} else {
-		g_return_if_fail (state->obj == NULL);
+		if (strcmp (role, GOG_BACKPLANE_OLD_ROLE_NAME) == 0)
+			res = gog_object_add_by_name (state->obj, GOG_BACKPLANE_NEW_ROLE_NAME, res);
+		else
+			res = gog_object_add_by_name (state->obj, role, res);
+	}
+	if (res != NULL) {
+		res->explicitly_typed_role = (type != NULL);
+		if (IS_GOG_PERSIST (res))
+			gog_persist_prep_sax (GOG_PERSIST (res), xin, attrs);
 	}
-
-	g_return_if_fail (res != NULL);
-
-	res->explicitly_typed_role = (type != NULL);
-	if (IS_GOG_PERSIST (res))
-		gog_persist_prep_sax (GOG_PERSIST (res), xin, attrs);
 
 	state->obj_stack = g_slist_prepend (state->obj_stack, state->obj);
 	state->obj = res;
@@ -586,7 +616,8 @@ gogo_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *unknown)
 {
 	GogXMLReadState *state = (GogXMLReadState *)xin->user_state;
 
-	if (state->obj_stack->data != NULL) {
+	/* Leave the top object on the stack for use in the caller */
+	if (state->obj_stack->next != NULL) {
 		state->obj = state->obj_stack->data;
 		state->obj_stack = g_slist_remove (state->obj_stack, state->obj);
 	} else
