@@ -27,6 +27,7 @@
 #include <goffice/goffice-config.h>
 #include "go-combo-box.h"
 #include <goffice/utils/go-marshalers.h>
+#include <goffice/gtk/goffice-gtk.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -791,10 +792,11 @@ go_combo_box_set_relief (GOComboBox *combo, GtkReliefStyle relief)
 }
 
 void
-go_combo_box_set_tooltip (GOComboBox *c, GtkTooltips *tips,
-			   char const *text, char const *priv_text)
+go_combo_box_set_tooltip (GOComboBox *c, G_GNUC_UNUSED void *tips,
+			  char const *text,
+			  G_GNUC_UNUSED char const *priv_text)
 {
-#warning this is ugly the tip moves as we jump from preview to arrow
-	gtk_tooltips_set_tip (tips, c->priv->display_widget, text, priv_text);
-	gtk_tooltips_set_tip (tips, c->priv->arrow_button, text, priv_text);
+/* FIXME FIXME FIXME this is ugly the tip moves as we jump from preview to arrow */
+	go_widget_set_tooltip_text (c->priv->display_widget, text);
+	go_widget_set_tooltip_text (c->priv->arrow_button, text);
 }
