@@ -24,6 +24,7 @@
 #include <goffice/app/go-cmd-context.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/goffice-priv.h>
+#include <goffice/gtk/go-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <atk/atkrelation.h>
@@ -103,15 +104,17 @@ go_gtk_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* sto
 }
 
 /**
- * go_libglade_new :
+ * go_glade_new :
  * @gcc : #GOCmdContext
  * @gladefile :
  *
  * Simple utility to open glade files
+ *
+ * 0.4.3 : renamed from go_libglade_new
  **/
 GladeXML *
-go_libglade_new (char const *gladefile, char const *root,
-		 char const *domain, GOCmdContext *gcc)
+go_glade_new (char const *gladefile, char const *root,
+	      char const *domain, GOCmdContext *gcc)
 {
 	GladeXML *gui;
 	char *f;
@@ -134,6 +137,66 @@ go_libglade_new (char const *gladefile, char const *root,
 	return gui;
 }
 
+/**
+ * go_glade_signal_connect :
+ * @gui : #GladeXML
+ * @instance_name :
+ * @detailed_signal :
+ * @c_handle : #GCallback
+ * @data :
+ *
+ * Convenience wrapper around g_signal_connect for glade.
+ **/
+gulong
+go_glade_signal_connect	(GladeXML	*gui,
+			 gchar const	*instance_name,
+			 gchar const	*detailed_signal,
+			 GCallback	 c_handler,
+			 gpointer	 data)
+{
+	GtkWidget *w;
+	g_return_val_if_fail (gui != NULL, 0);
+	w = glade_xml_get_widget (gui, instance_name);
+	g_return_val_if_fail (w != NULL, 0);
+	return g_signal_connect (w, detailed_signal, c_handler, data);
+}
+
+/**
+ * go_glade_signal_connect :
+ * @gui : #GladeXML
+ * @instance_name :
+ * @detailed_signal :
+ * @c_handle : #GCallback
+ * @data :
+ *
+ * Convenience wrapper around g_signal_connect_swapped for glade.
+ **/
+gulong
+go_glade_signal_connect_swapped (GladeXML	*gui,
+				 gchar const	*instance_name,
+				 gchar const	*detailed_signal,
+				 GCallback	 c_handler,
+				 gpointer	 data)
+{
+	GtkWidget *w;
+	g_return_val_if_fail (gui != NULL, 0);
+	w = glade_xml_get_widget (gui, instance_name);
+	g_return_val_if_fail (w != NULL, 0);
+	return g_signal_connect_swapped (w, detailed_signal, c_handler, data);
+}
+
+
+/*
+ * A variant of gtk_window_activate_default that does not end up reactivating
+ * the widget that [Enter] was pressed in.
+ */
+static void
+cb_activate_default (GtkWindow *window)
+{
+	if (window->default_widget && GTK_WIDGET_IS_SENSITIVE (window->default_widget))
+		gtk_widget_activate (window->default_widget);
+}
+
 /**
  * go_gtk_editable_enters:
  * @window:
@@ -151,31 +214,7 @@ go_gtk_editable_enters (GtkWindow *window, GtkWidget *w)
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	g_signal_connect_swapped (G_OBJECT (w),
 		"activate",
-		G_CALLBACK (gtk_window_activate_default), window);
-}
-
-GdkPixbuf *
-go_pixbuf_intelligent_scale (GdkPixbuf *buf, guint width, guint height)
-{
-	GdkPixbuf *scaled;
-	int w, h;
-	unsigned long int ow = gdk_pixbuf_get_width (buf);
-	unsigned long int oh = gdk_pixbuf_get_height (buf);
-
-	if (ow > width || oh > height) {
-		if (ow * height > oh * width) {
-			w = width;
-			h = width * (((double)oh)/(double)ow);
-		} else {
-			h = height;
-			w = height * (((double)ow)/(double)oh);
-		}
-
-		scaled = gdk_pixbuf_scale_simple (buf, w, h, GDK_INTERP_BILINEAR);
-	} else
-		scaled = g_object_ref (buf);
-
-	return scaled;
+		G_CALLBACK (cb_activate_default), window);
 }
 
 void
@@ -225,17 +264,19 @@ cb_parent_mapped (GtkWidget *parent, GtkWindow *window)
 void
 go_gtk_window_set_transient (GtkWindow *toplevel, GtkWindow *window)
 {
-/* FIXME:                                                                     */
-/* 	GtkWindowPosition position = gnome_preferences_get_dialog_position(); */
+#if 0
+	GtkWindowPosition position = gnome_preferences_get_dialog_position(); */
+	if (position == GTK_WIN_POS_NONE)
+		position = GTK_WIN_POS_CENTER_ON_PARENT;
+#else
 	GtkWindowPosition position = GTK_WIN_POS_CENTER_ON_PARENT;
+#endif
 
 	g_return_if_fail (GTK_IS_WINDOW (toplevel));
 	g_return_if_fail (GTK_IS_WINDOW (window));
 
 	gtk_window_set_transient_for (window, toplevel);
 
-	if (position == GTK_WIN_POS_NONE)
-		position = GTK_WIN_POS_CENTER_ON_PARENT;
 	gtk_window_set_position (window, position);
 
 	if (!GTK_WIDGET_MAPPED (toplevel))
@@ -497,20 +538,6 @@ go_gtk_select_image (GtkWindow *toplevel, const char *initial)
 	return uri;
 }
 
-/**
- * go_gui_get_image_save_info:
- * @toplevel: a #GtkWindow
- * @supported_formats: a #GSList of supported file formats
- * @ret_format: default file format
- * @resolution: export resolution
- *
- * Opens a file chooser and let user choose file URI and format in a list of
- * supported ones.
- *
- * returns: file URI string, file #GOImageFormat stored in @ret_format, and
- * export resolution in @resolution.
- **/
-
 typedef struct {
 	char *uri;
 	double resolution;
@@ -536,6 +563,19 @@ cb_format_combo_changed (GtkComboBox *combo, GtkWidget *expander)
 				  format_info->is_dpi_useful);
 }
 
+/**
+ * go_gui_get_image_save_info:
+ * @toplevel: a #GtkWindow
+ * @supported_formats: a #GSList of supported file formats
+ * @ret_format: default file format
+ * @resolution: export resolution
+ *
+ * Opens a file chooser and let user choose file URI and format in a list of
+ * supported ones.
+ *
+ * Returns: file URI string, file #GOImageFormat stored in @ret_format, and
+ * 	export resolution in @resolution.
+ **/
 char *
 go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			 GOImageFormat *ret_format, double *resolution)
@@ -641,7 +681,7 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			g_free (new_uri);
 			g_free (uri);
 			uri = NULL;
-			goto out;
+			goto loop;
 		} else {
 			g_free (uri);
 			uri = new_uri;
@@ -963,134 +1003,49 @@ go_gtk_query_yes_no (GtkWindow *parent, gboolean default_answer,
 		go_gtk_dialog_run (GTK_DIALOG (dialog), parent);
 }
 
-/**
- * go_pixbuf_new_from_file :
- * @filename :
- *
- * utility routine to create pixbufs from file @name in the goffice_icon_dir.
- *
- * Returns a GdkPixbuf that the caller is responsible for.
- **/
-GdkPixbuf *
-go_pixbuf_new_from_file (char const *filename)
+#ifndef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
+static GtkTooltips *
+get_tooltips (GObject *obj)
 {
-	char *path = g_build_filename (go_sys_icon_dir (), filename, NULL);
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	g_free (path);
-	return pixbuf;
-}
+	GtkTooltips *tips = g_object_get_data (obj, "-go-tips");
 
-#ifndef HAVE_GDK_CAIRO_SET_SOURCE_PIXBUF 
-/* This is public from 2.8 onwards.   */
-static void
-gdk_cairo_set_source_pixbuf (cairo_t   *cr,
-			     GdkPixbuf *pixbuf,
-			     double     pixbuf_x,
-			     double     pixbuf_y)
-{
-  gint width = gdk_pixbuf_get_width (pixbuf);
-  gint height = gdk_pixbuf_get_height (pixbuf);
-  guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
-  int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-  guchar *cairo_pixels;
-  cairo_format_t format;
-  cairo_surface_t *surface;
-  static const cairo_user_data_key_t key;
-  int j;
+	if (!tips) {
+		tips = gtk_tooltips_new ();
 
-  if (n_channels == 3)
-    format = CAIRO_FORMAT_RGB24;
-  else
-    format = CAIRO_FORMAT_ARGB32;
-
-  cairo_pixels = g_malloc (4 * width * height);
-  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
-						 format,
-						 width, height, 4 * width);
-  cairo_surface_set_user_data (surface, &key,
-			       cairo_pixels, (cairo_destroy_func_t)g_free);
-
-  for (j = height; j; j--)
-    {
-      guchar *p = gdk_pixels;
-      guchar *q = cairo_pixels;
-
-      if (n_channels == 3)
-	{
-	  guchar *end = p + 3 * width;
-	  
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      q[0] = p[2];
-	      q[1] = p[1];
-	      q[2] = p[0];
-#else	  
-	      q[1] = p[0];
-	      q[2] = p[1];
-	      q[3] = p[2];
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+		g_object_ref_sink (tips);
+#else
+		g_object_ref (tips);
+		gtk_object_sink (GTK_OBJECT (tips));
 #endif
-	      p += 3;
-	      q += 4;
-	    }
-	}
-      else
-	{
-	  guchar *end = p + 4 * width;
-	  guint t1,t2,t3;
-	    
-#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
-
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      MULT(q[0], p[2], p[3], t1);
-	      MULT(q[1], p[1], p[3], t2);
-	      MULT(q[2], p[0], p[3], t3);
-	      q[3] = p[3];
-#else	  
-	      q[0] = p[3];
-	      MULT(q[1], p[0], p[3], t1);
-	      MULT(q[2], p[1], p[3], t2);
-	      MULT(q[3], p[2], p[3], t3);
-#endif
-	      
-	      p += 4;
-	      q += 4;
-	    }
-	  
-#undef MULT
+		g_object_set_data_full (obj, "-go-tips", tips,
+					(GDestroyNotify)g_object_unref);
 	}
 
-      gdk_pixels += gdk_rowstride;
-      cairo_pixels += 4 * width;
-    }
-
-  cairo_set_source_surface (cr, surface, pixbuf_x, pixbuf_y);
-  cairo_surface_destroy (surface);
+	return tips;
 }
 #endif
 
-/**
- * go_cairo_set_source_pixbuf:
- * @cr: a #Cairo context
- * @pixbuf: a #GdkPixbuf
- * @pixbuf_x: X coordinate of location to place upper left corner of @pixbuf
- * @pixbuf_y: Y coordinate of location to place upper left corner of @pixbuf
- * 
- * Sets the given pixbuf as the source pattern for the Cairo context.
- * The pattern has an extend mode of %CAIRO_EXTEND_NONE and is aligned
- * so that the origin of @pixbuf is @pixbuf_x, @pixbuf_y
- *
- **/
 void
-go_cairo_set_source_pixbuf (cairo_t   *cr,
-			    GdkPixbuf *pixbuf,
-			    double     pixbuf_x,
-			    double     pixbuf_y)
+go_widget_set_tooltip_text (GtkWidget *widget, const gchar *tip)
 {
-	gdk_cairo_set_source_pixbuf (cr, pixbuf, pixbuf_x, pixbuf_y);
+#ifdef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
+	gtk_widget_set_tooltip_text (widget, tip);
+#else
+	GtkTooltips *tips = get_tooltips (G_OBJECT (widget));
+	gtk_tooltips_set_tip (tips, widget, tip, NULL);
+#endif
+}
+
+void
+go_tool_item_set_tooltip_text (GtkToolItem *item, const gchar *tip)
+{
+#ifdef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
+	gtk_tool_item_set_tooltip_text (item, tip);
+#else
+	GtkTooltips *tips = get_tooltips (G_OBJECT (item));
+	gtk_tool_item_set_tooltip (item, tips, tip, NULL);
+#endif
 }
 
 
@@ -1182,3 +1137,66 @@ go_dialog_guess_alternative_button_order (GtkDialog *dialog)
 								    new_order);
 	g_free (new_order);
 }
+
+/**
+ * go_menu_position_below:
+ * @menu: a #GtkMenu
+ * @x:
+ * @y:
+ * @push_in:
+ * @user_data:
+ *
+ * Implementation of a GtkMenuPositionFunc that positions
+ * the child window under the parent one, for use with gtk_menu_popup.
+ **/
+
+void
+go_menu_position_below (GtkMenu  *menu,
+			gint     *x,
+			gint     *y,
+			gint     *push_in,
+			gpointer  user_data)
+{
+	GtkWidget *widget = GTK_WIDGET (user_data);
+	gint sx, sy;
+	GtkRequisition req;
+	GdkScreen *screen;
+	gint monitor_num;
+	GdkRectangle monitor;
+
+	gdk_window_get_origin (widget->window, &sx, &sy);
+
+	if (GTK_WIDGET_NO_WINDOW (widget))
+	{
+		sx += widget->allocation.x;
+		sy += widget->allocation.y;
+	}
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &req);
+
+	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
+		*x = sx;
+	else
+		*x = sx + widget->allocation.width - req.width;
+	*y = sy;
+
+	screen = gtk_widget_get_screen (widget);
+	monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+	if (*x < monitor.x)
+		*x = monitor.x;
+	else if (*x + req.width > monitor.x + monitor.width)
+		*x = monitor.x + monitor.width - req.width;
+
+	if (monitor.y + monitor.height - *y - widget->allocation.height >= req.height)
+		*y += widget->allocation.height;
+	else if (*y - monitor.y >= req.height)
+		*y -= req.height;
+	else if (monitor.y + monitor.height - *y - widget->allocation.height > *y - monitor.y)
+		*y += widget->allocation.height;
+	else
+		*y -= req.height;
+
+	*push_in = FALSE;
+}
