@@ -39,7 +39,7 @@
 #include "go-color.h"
 #include "datetime.h"
 #include "go-glib-extras.h"
-#include "go-math.h"
+#include <goffice/math/go-math.h>
 #include <glib/gi18n-lib.h>
 
 #include <time.h>
@@ -282,7 +282,11 @@ struct _GOFormat {
 			unsigned int E_format    : 1;
 			unsigned int use_markup  : 1;
 			unsigned int has_date    : 1;
+			unsigned int date_ybm    : 1;  /* year, then month.  */
+			unsigned int date_mbd    : 1;  /* month, then day.  */
+			unsigned int date_dbm    : 1;  /* day, then month.  */
 			unsigned int has_time    : 1;
+			unsigned int has_hour    : 1;
 			unsigned int fraction    : 1;
 			unsigned int scale_is_2  : 1;
 			unsigned int has_general : 1;
@@ -426,10 +430,10 @@ static long double beyond_precisionl;
 static GHashTable *style_format_hash = NULL;
 
 /* used to generate formats when delocalizing so keep the leadings caps */
-static const struct {
+static struct {
 	char const *name;
 	GOColor	go_color;
-} format_colors[] = {
+} const format_colors[] = {
 	{ N_("Black"),	 RGBA_BLACK },
 	{ N_("Blue"),	 RGBA_BLUE },
 	{ N_("Cyan"),	 RGBA_CYAN },
@@ -441,18 +445,21 @@ static const struct {
 };
 
 /*
- * Parse [CcolorName].
+ * go_format_parse_color :
+ * @str :
+ * @color :
+ * @n :
+ * @named :
  *
- * Return TRUE, if ok.  Then @color will be filled in, and @n will be
- * a number 0-7 for standard colors.
- *
- * Returns FALSE otherwise and @color will be zeroed.
- */
+ * Return: TRUE, if ok.  Then @color will be filled in, and @n will be
+ * 	a number 0-7 for standard colors.
+ *	Returns FALSE otherwise and @color will be zeroed.
+ **/
 static gboolean
-go_format_parse_color (const char *str, GOColor *color,
+go_format_parse_color (char const *str, GOColor *color,
 		       int *n, gboolean *named)
 {
-	const char *close;
+	char const *close;
 	unsigned int ui;
 
 	*color = 0;
@@ -1042,6 +1049,9 @@ go_format_parse_sequential (const char *str, GString *prg,
 {
 	int date_decimals = 0;
 	gboolean seen_date = FALSE;
+	gboolean seen_year = FALSE;
+	gboolean seen_month = FALSE;
+	gboolean seen_day = FALSE;
 	gboolean seen_time = FALSE;
 	gboolean seen_hour = FALSE;
 	gboolean seen_ampm = FALSE;
@@ -1050,6 +1060,9 @@ go_format_parse_sequential (const char *str, GString *prg,
 	gboolean seen_elapsed = FALSE;
 	gboolean m_is_minutes = FALSE;
 	gboolean seconds_trigger_minutes = TRUE;
+	gboolean date_ybm = FALSE;
+	gboolean date_mbd = FALSE;
+	gboolean date_dbm = FALSE;
 
 	if (!prg)
 		prg = g_string_new (NULL);
@@ -1074,6 +1087,11 @@ go_format_parse_sequential (const char *str, GString *prg,
 			case 3: ADD_OP (OP_DATE_WEEKDAY_3); break;
 			default: ADD_OP (OP_DATE_WEEKDAY); break;
 			}
+			if (!seen_day) {
+				seen_day = TRUE;
+				if (seen_month)
+					date_mbd = TRUE;
+			}
 			break;
 		}
 
@@ -1082,6 +1100,7 @@ go_format_parse_sequential (const char *str, GString *prg,
 			while (*str == 'y' || *str == 'Y')
 				str++, n++;
 			seen_date = TRUE;
+			seen_year = TRUE;
 			ADD_OP (n <= 2 ? OP_DATE_YEAR_2 : OP_DATE_YEAR);
 			break;
 		}
@@ -1091,6 +1110,7 @@ go_format_parse_sequential (const char *str, GString *prg,
 			while (*str == 'b' || *str == 'B')
 				str++, n++;
 			seen_date = TRUE;
+			seen_year = TRUE;
 			ADD_OP (n <= 2 ? OP_DATE_YEAR_THAI_2 : OP_DATE_YEAR_THAI);
 			break;
 		}
@@ -1098,6 +1118,7 @@ go_format_parse_sequential (const char *str, GString *prg,
 		case 'e':
 			while (*str == 'e') str++;
 			seen_date = TRUE;
+			seen_year = TRUE;
 			ADD_OP (OP_DATE_YEAR);
 			break;
 
@@ -1138,6 +1159,13 @@ go_format_parse_sequential (const char *str, GString *prg,
 				case 5: ADD_OP (OP_DATE_MONTH_NAME_1); break;
 				default: ADD_OP (OP_DATE_MONTH_NAME); break;
 				}
+				if (!seen_month) {
+					seen_month = TRUE;
+					if (seen_day)
+						date_dbm = TRUE;
+					if (seen_year && !seen_day)
+						date_ybm = TRUE;
+				}
 			}
 			break;
 		}
@@ -1256,6 +1284,9 @@ go_format_parse_sequential (const char *str, GString *prg,
 		if (seen_date) {
 			*p++ = OP_DATE_SPLIT;
 			fmt->u.number.has_date = TRUE;
+			fmt->u.number.date_ybm = date_ybm;
+			fmt->u.number.date_mbd = date_mbd;
+			fmt->u.number.date_dbm = date_dbm;
 		}
 		if (seen_time) {
 			guchar op;
@@ -1273,6 +1304,7 @@ go_format_parse_sequential (const char *str, GString *prg,
 			}
 			*p++ = op;
 			fmt->u.number.has_time = TRUE;
+			fmt->u.number.has_hour = seen_hour;
 		}
 		if (pstate->has_general) {
 			ADD_OP (OP_NUM_GENERAL_DO);
@@ -1526,6 +1558,8 @@ go_format_parse_number_E (GOFormatParseState *pstate)
 
 		simplify_mantissa = TRUE;
 		for (i = 0; i < pstate->tno_E; i++) {
+			if (GET_TOKEN(i).token == '.')
+				break;
 			if (GET_TOKEN(i).token == '0') {
 				simplify_mantissa = FALSE;
 				break;
@@ -2153,16 +2187,14 @@ SUFFIX(go_format_execute) (PangoLayout *layout, GString *dst,
 					      date_conv);
 			if (!g_date_valid (&date)) {
 				res = GO_FORMAT_NUMBER_DATE_ERROR;
-				break;
+				g_date_set_dmy (&date, 1, 1, 1900);
 			}
 			year = g_date_get_year (&date);
-			if (year > 9999) {
-				res = GO_FORMAT_NUMBER_DATE_ERROR;
-				break;
-			}
 			month = g_date_get_month (&date);
 			day = g_date_get_day (&date);
 			weekday = g_date_get_weekday (&date);
+			if (year > 9999)
+				res = GO_FORMAT_NUMBER_DATE_ERROR;
 			break;
 		}
 
@@ -3441,7 +3473,7 @@ go_format_str_localize (char const *str)
  *
  * Increaseds the displayed precision for @fmt by one digit.
  *
- * Returns NULL if the new format would not change things
+ * Returns: NULL if the new format would not change things
  **/
 GOFormat *
 go_format_inc_precision (GOFormat const *fmt)
@@ -3496,8 +3528,19 @@ go_format_inc_precision (GOFormat const *fmt)
 			break;
 		}
 
+		case 'E': 
+			if (last_zero != -2) {
+				if (last_zero>=0)
+					g_string_insert_len (res, last_zero + 1, ".0",2);
+				else
+					g_string_append (res, ".0");
+				last_zero = -2;
+			}
+			g_string_append_c (res, t);
+			break;
+			
 		case '0':
-			if (last_zero == -1)
+			if (last_zero != -2)
 				last_zero = res->len;
 			/* Fall through.  */
 
@@ -3513,7 +3556,7 @@ go_format_inc_precision (GOFormat const *fmt)
  *
  * Decreases the displayed precision for @fmt by one digit.
  *
- * Returns NULL if the new format would not change things
+ * Returns: NULL if the new format would not change things
  **/
 GOFormat *
 go_format_dec_precision (GOFormat const *fmt)
@@ -3973,19 +4016,35 @@ go_format_unref (GOFormat *gf)
 #endif
 
 
+#ifdef DEFINE_COMMON
+/**
+ * go_format_is_invalid
+ * @fmt: Format to query
+ *
+ * Returns: TRUE if, and if only, the format is invalid
+ **/
+gboolean
+go_format_is_invalid (GOFormat const *fmt)
+{
+	g_return_val_if_fail (fmt != NULL, TRUE);
+	return fmt->typ == GO_FMT_INVALID;
+}
+#endif
+
 #ifdef DEFINE_COMMON
 /**
  * go_format_is_general
  * @fmt: Format to query
  *
- * Returns TRUE if the format is "General", possibly with condition,
- * color, and/or locale.  ("xGeneral" is thus not considered to be General
- * for the purpose of this function.)
- * Returns FALSE otherwise.
- */
+ * Returns: TRUE if the format is "General", possibly with condition,
+ * 	color, and/or locale.  ("xGeneral" is thus not considered to be General
+ * 	for the purpose of this function.)
+ *	Returns FALSE otherwise.
+ **/
 gboolean
 go_format_is_general (GOFormat const *fmt)
 {
+	g_return_val_if_fail (fmt != NULL, FALSE);
 	return fmt->typ == GO_FMT_NUMBER && fmt->u.number.is_general;
 }
 #endif
@@ -3995,12 +4054,13 @@ go_format_is_general (GOFormat const *fmt)
  * go_format_is_markup
  * @fmt: Format to query
  *
- * Returns TRUE if the format is a markup format
- * Returns FALSE otherwise.
+ * Returns: TRUE if the format is a markup format
+ * 	Returns FALSE otherwise.
  */
 gboolean
 go_format_is_markup (GOFormat const *fmt)
 {
+	g_return_val_if_fail (fmt != NULL, FALSE);
 	return fmt->typ == GO_FMT_MARKUP;
 }
 #endif
@@ -4010,12 +4070,13 @@ go_format_is_markup (GOFormat const *fmt)
  * go_format_is_text
  * @fmt: Format to query
  *
- * Returns TRUE if the format is a text format
- * Returns FALSE otherwise.
- */
+ * Returns: TRUE if the format is a text format
+ * 	Returns FALSE otherwise.
+ **/
 gboolean
 go_format_is_text (GOFormat const *fmt)
 {
+	g_return_val_if_fail (fmt != NULL, FALSE);
 	return fmt->typ == GO_FMT_TEXT;
 }
 #endif
@@ -4025,9 +4086,9 @@ go_format_is_text (GOFormat const *fmt)
  * go_format_is_var_width
  * @fmt: Format to query
  *
- * Returns TRUE if the format is variable width, i.e., can stretch.
- * Returns FALSE otherwise.
- */
+ * Returns: TRUE if the format is variable width, i.e., can stretch.
+ * 	Returns FALSE otherwise.
+ **/
 gboolean
 go_format_is_var_width (GOFormat const *fmt)
 {
@@ -4057,10 +4118,10 @@ go_format_is_var_width (GOFormat const *fmt)
  * go_format_is_date:
  * @fmt: Format to query
  *
- * Returns TRUE if the format is a date format.
- * Returns FALSE if the format is not a date format.
- * Returns -1 if the format is inconsistent.
- */
+ * Returns: TRUE if the format is a date format.
+ * 	Returns FALSE if the format is not a date format.
+ * 	Returns -1 if the format is inconsistent.
+ **/
 int
 go_format_is_date (GOFormat const *fmt)
 {
@@ -4069,6 +4130,55 @@ go_format_is_date (GOFormat const *fmt)
 }
 #endif
 
+#ifdef DEFINE_COMMON
+/**
+ * go_format_month_before_day:
+ * @fmt: Format to query
+ *
+ * Returns:
+ *  0, if format is a date format with day and month in that order
+ *  1, if format is a date format with month and day in that order, unless
+ *  2, if format is a date with year before month before day
+ * -1, otherwise.
+ **/
+int
+go_format_month_before_day (GOFormat const *fmt)
+{
+	g_return_val_if_fail (fmt != NULL, -1);
+
+	if (go_format_is_date (fmt) != 1)
+		return -1;
+	if (fmt->u.number.date_ybm)
+		return +2;
+	if (fmt->u.number.date_mbd)
+		return +1;
+	if (fmt->u.number.date_dbm)
+		return 0;
+	return -1;
+}
+#endif
+
+
+#ifdef DEFINE_COMMON
+/**
+ * go_format_has_hour:
+ * @fmt: Format to query
+ *
+ * Returns: TRUE if format is a number format with an hour specifier
+ * 	    FALSE otherwise.
+ **/
+gboolean
+go_format_has_hour (GOFormat const *fmt)
+{
+	g_return_val_if_fail (fmt != NULL, FALSE);
+
+	return (fmt->typ == GO_FMT_NUMBER &&
+		fmt->u.number.has_time &&
+		fmt->u.number.has_hour);
+}
+#endif
+
+
 const GOFormat *
 SUFFIX(go_format_specialize) (GOFormat const *fmt, DOUBLE val, char type,
 			      gboolean *inhibit_minus)
