@@ -32,7 +32,7 @@
 #include <string.h>
 
 
-char const *grad_dir_names[] = {
+static char const *grad_dir_names[] = {
 	"n-s",
 	"s-n",
 	"n-s-mirrored",
@@ -68,40 +68,6 @@ go_gradient_dir_as_str (GOGradientDirection dir)
 		: grad_dir_names[dir];
 }
 
-#ifdef GOFFICE_WITH_GTK
-GtkWidget *
-go_gradient_selector (GOColor start, GOColor end)
-{
-	int const W = 20, H = 20;
-	unsigned	 i;
-	GOComboPixmaps	*w;
-	GdkPixbuf	*pixbuf;
-	ArtRender	*render;
-	ArtGradientLinear gradient;
-	ArtGradientStop	  stops[2];
-
-	w = go_combo_pixmaps_new (4);
-	for (i = 0; i < GO_GRADIENT_MAX; i++) {
-		GOGradientDirection dir = (GOGradientDirection)i;
-		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, W, H);
-		gdk_pixbuf_fill (pixbuf, 0); /* in case the fill colours have alpha = 0 */
-		render = art_render_new (0, 0, W, H,
-			gdk_pixbuf_get_pixels (pixbuf),
-			gdk_pixbuf_get_rowstride (pixbuf),
-			gdk_pixbuf_get_n_channels (pixbuf) - 1,
-			8, ART_ALPHA_SEPARATE, NULL);
-		go_gradient_setup (&gradient, dir, start, end, 0, 0,
-			W, H, stops);
-		art_render_gradient_linear (render,
-			&gradient, ART_FILTER_NEAREST);
-		art_render_invoke (render);
-		go_combo_pixmaps_add_element (w, pixbuf, dir, NULL);
-	}
-
-	return GTK_WIDGET (w);
-}
-#endif /* GOFFICE_WITH_GTK */
-
 void
 go_gradient_setup (ArtGradientLinear *gradient,
 		   GOGradientDirection dir, GOColor col0, GOColor col1,
